module SinatraHelpers
  def user_info(users, env=env, current_user=nil)
    users.map do |user|
      # For pending users, check that they're not expired
      if user.status == "pending"
        activation = Activation.find_by_user_id(user.id)
        unless fresh?(activation)
          user.status = "expired"
          user.save!
          ConsoleAuth.logger.info("The activation for user #{user.id} (#{user.username}) has expired. They will need to be deleted and re-added.")
        end
      end

      # For first implementation each user only has one role, so just
      # take the first one
      result = { "id"       => user.id,
               "username"   => user.username,
               "role"       => user.roles.first.name,
               "origin"     => user_origin_description(user.origin),
               "created_at" => user.created_at.strftime("%m/%d/%Y %H:%M"),
               "status"     => user.status }
      result['activation_link'] = activation_link(activation.key, env['rack.url_scheme'], env['SERVER_PORT']) if activation
      result['current_admin'] = "true" if current_user && user.id == current_user.id
      result
    end
  end

  def user_origin_description(origin)
    origin_config = ConsoleAuth.authenticator_details(origin)
    if origin_config == nil
      origin_config =  {
        'description' => 'Unknown Source of Authentication...'
      }
    end
    origin_config['description']
  end

  def activation_link(activation_key, scheme, server_port)
    port = ":#{server_port}" unless server_port.nil?
    "#{scheme}://#{ConsoleAuth.get_config['authentication']['console_hostname']}#{port}/auth/activate/#{activation_key}"
  end

  def create_activation(user, admin_email)
    begin
      keysrc = [('a'..'z'),('A'..'Z'),('0'..'9')].map{|i| i.to_a}.flatten
      activation_key = (0..32).map{ keysrc[rand(keysrc.length)]  }.join
      activation = Activation.new(:key => activation_key, :user_id => user.id, :admin_email => admin_email)
      activation.save!
    rescue Exception => e
      ConsoleAuth.logger.error("Creating activation record for user #{user.id} failed; #{e.inspect}")
      return nil
    end
    activation
  end

  def send_confirmation_email(user, admin, activation_time, activation_url)
    t = Thread.new do
      begin

        content = <<-EMAIL
An account has been created to allow you access to the Puppet Enterprise
console. To complete the account registration process, visit this link:
#{activation_url}

        EMAIL

        freshness_period = ConsoleAuth.get_config['activation']['freshness_period']
        if freshness_period != 0
          expiration = activation_time + (freshness_period * 86400)
          content += <<-EMAIL2
For security reasons, this link will expire on #{expiration.strftime("%m/%d/%Y")} at #{expiration.strftime("%H:%M:%S %Z")}.

          EMAIL2
        end

        content += <<-EMAIL3
If you have questions or have received this notification in error,
contact your system administrator or IT department.
        EMAIL3
        require 'mail'
        smtp_config = ConsoleAuth.get_config['smtp']
        smtp_conn = Net::SMTP.new(smtp_config['address'], smtp_config['port'])
        smtp_conn.enable_starttls if smtp_config['use_tls']
        smtp_conn.start(smtp_config['address'], smtp_config['username'], smtp_config['password'], :plain)
        Mail.defaults do
          delivery_method :smtp_connection, { :connection => smtp_conn }
        end

        mail = Mail.new do
          from    admin
          to      user.username
          subject 'Puppet Enterprise Console Account Confirmation'
          body    content
        end
        mail.deliver!
        ConsoleAuth.logger.info("User #{user.id} was sent a confirmation email at #{user.username} from #{admin}; status was set to pending.")
        smtp_conn.finish
      rescue Exception => e
        ConsoleAuth.logger.error("User #{user.id} could not be sent a confirmation email at #{user.username}: #{e.inspect}")
      end
    end
  end

  def fresh?(activation)
    freshness_period = ConsoleAuth.get_config['activation']['freshness_period']
    return true if freshness_period == 0
    activation.created_at >= (Time.now - (freshness_period * 86400))
  end
end

